function RobotSimulator(InitialPose, LRF, MapName, GridMappingFunction)
% This files is a simple simulation of a differential drive point-like robot
% randomly moving in a 2-D world. A simulated laser range finder is used to
% compute scanlines in this world.
%
% (c) Philippe Giguere 2013
%     philippe.giguere@ift.ulaval.ca
%

% load a world map.
world2 = imread(MapName);
world = world2(:,:,1);

DetectCollision = true;
Collision = false;

% Robot initialization
x = InitialPose(1);     % x location, in pixel
y = InitialPose(2);     % y location, in pixel
Heading = InitialPose(3); % Heading of the robot, in rad. 0 is along x-axis
% V = 1;       % Velocity, in pixel per iteration
Speed = 10;  % speed for forward motion

% Simulation parameter
dT = 1;  % time interval.

clf 
subplot(2,1,1)

% Display the world map
ShowWorldMap(world2);
hold on;
plot(x,y,'o');
line([x x+20*cos(Heading)],[y y+20*sin(Heading)]); % To show heading


% Each iteration is a click of the user for a straight path. Note that
% there is no obstacle detection so far: it will just drive through the
% obstacles.
IterationNum = 0;

while (IterationNum<20) 
    
    display('Click on the screen for the next waypoint');
    [Xtarget Ytarget] = ginput(1);
    
    % Computing a simple trajectory to follow in straight line
    Trajectory(:,IterationNum+1) = [Xtarget; Ytarget];
    StepSize = 0.1;
    U = []; % The command vector, with the first dimension as forward velocity (m/s) and
            % second dimension the rotational velocity (rad/s)
    Straight = [];
    DesiredHeading = atan2(Ytarget-y,Xtarget-x);
    DeltaHeading = wrapToPi(DesiredHeading-Heading);
    NRotStep = ceil(abs(DeltaHeading)/0.4);
    U(1,1:NRotStep) = 0.0;
    U(2,1:NRotStep) = DeltaHeading/NRotStep; % To make sure we have bang-on rotation;
    
    Distance = sqrt((Xtarget-x)^2+(Ytarget-y)^2);
    NStraight = ceil(Distance/Speed);
    Straight(1,1:NStraight) = Speed;
    Straight(2,1:NStraight) = 0; 
    
    % The complete command vector for this particular waypoint.
    U = cat(2,U,Straight);

    for cmd = 1:size(U,2)
        % Always turn, irrespective of collisions.
        Heading = Heading + U(2,cmd)*dT;
        
        if (DetectCollision)
            % Integrating motion of the differential drive robot
            xTmp = x + U(1,cmd)*cos(Heading)*dT;
            yTmp = y + U(1,cmd)*sin(Heading)*dT;
            
            % Checking for collision
            Collision = BresenhamLine(x, xTmp, y, yTmp, world);
            if (~Collision)
                x = xTmp;
                y = yTmp;
            else
                display('Detected Collision!');
            end
        else
            % Integrating motion of the differential drive robot
            x = x + U(1,cmd)*cos(Heading)*dT;
            y = y + U(1,cmd)*sin(Heading)*dT;
        end
        
        clf 
        subplot(2,1,1)
        % Display the world map
        ShowWorldMap(world2);
        hold on;

        % Get a laser scan of the environment.
        [ScanLine] = GetLaserScan(x,y,Heading,world,LRF);
        hold on;
        plot(x,y,'o'); % robot position
        line([x x+20*cos(Heading)],[y y+20*sin(Heading)]); % To show heading
    
        subplot(2,2,3);
        polar(ScanLine(1,:),ScanLine(2,:),'*');
        xlabel('Angle (rad)');
        ylabel('Range');
        
        GridMappingFunction([x y Heading],ScanLine);
        
        drawnow(); % This forces a redraw of the screen.

        if (Collision)
            break;
        end
    end
end